using System;
using System.ComponentModel;
using System.Xml.Serialization;

using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace EQIA.EQIDL
{
    /// <summary>
    /// A gui component that contains a filename for a texture resource, also stores size information
    /// about the texture.
    /// </summary>
    public class TextureInfo : Class, ILoadable, IDrawable
    {
        private Texture2D myTexture;
        private Size mySize;

        /// <summary>
        /// Size of the texture, if this size isn't the actual size of the texture, it will be scaled
        /// to this size.
        /// </summary>
        [XmlElement("Size", IsNullable = false)]
        [Browsable(true), Description("Size of the texture, if this size isn't the actual size of the texture, it will be scaled to this size.")]
        public Size Size { get { return mySize; } set { mySize = value; } }

        public TextureInfo()
            : this(null)
        {
        }

        public TextureInfo(EQInterface eqInterface)
            : base(eqInterface)
        {
            mySize = Size.Empty;
        }

        protected override void SetProperties(object obj)
        {
            base.SetProperties(obj);

            TextureInfo temp_TextureInfo = obj as TextureInfo;
            if(temp_TextureInfo != null)
                Size = temp_TextureInfo.Size;
        }

        #region ILoadable Members

        [XmlIgnore()]
#if DEBUG
        [Browsable(true), Category("Internal Data"), NotifyParentProperty(true)]
#else
        [Browsable(false)]
#endif
        public bool IsLoaded { get { return myTexture != null && !myTexture.IsDisposed; } }

        public void LoadContent(GraphicsDevice device)
        {
            if (!string.IsNullOrEmpty(Name))
            {
                try
                {
                    // If there isn't a size, attempt to load the texture, it may fail though since some
                    //  graphic cards are incapable of loading textures outside the powers of two
                    myTexture = (mySize.Width > 0 && mySize.Height > 0) ?
                            Texture2D.FromFile(device, Interface.Path + "\\" + Name, mySize.Width, mySize.Height) :
                            Texture2D.FromFile(device, Interface.Path + "\\" + Name);
                }
                catch (Exception)
                {
                    /* There are a number of reasons this may fail
                     *  ArguementException - The filename is empty or contains invalid path chars (see System.IO.Path.InvalidPathChars)
                     *  ArguementNullException - The filename is null
                     *  PathTooLongException - The path is larger than 248 chars
                     *  DirectoryNotFoundException - Self explanatory
                     *  UnauthorizedAccessException - The user that executed the applications doesn't have permission to access the file
                     *  FileNotFoundException - Self explanatory
                     *  NotSupportedException - Either the filename or the image file is in an invalid format
                     */
                }
            }
        }

        public void UnloadContent()
        {
            if (myTexture != null)
            {
                if (!myTexture.IsDisposed)
                    myTexture.Dispose();

                myTexture = null;
            }
        }

        #endregion

        #region IDrawable Members

        /// <summary>
        /// Draw this texture.
        /// </summary>
        /// <param name="batch">The batch to draw this texture with.</param>
        public void Draw(SpriteBatch batch) { Draw(batch, new Rectangle(0, 0, mySize.Width, mySize.Height), null, RGB.White); }

        /// <summary>
        /// Draw this texture.
        /// </summary>
        /// <param name="batch">The batch to draw this texture with.</param>
        /// <param name="destination">A rectangle defining the destination area of this texture.</param>
        /// <param name="source">A rectangle defining the area of this texture to draw. Using null will draw the entire texture.</param>
        /// <param name="color">A color to apply as a tint to the texture.</param>
        public void Draw(SpriteBatch batch, Rectangle destination, Nullable<Rectangle> source, RGB color)
        {
            if (IsLoaded && destination.Width > 0 && destination.Height > 0)
            {
                if (source.HasValue && (source.Value.Width < 1 || source.Value.Height < 1)) return;
                batch.Draw(myTexture, destination, source, new Color(color.Red, color.Green, color.Blue, color.Alpha));
            }
        }

        #endregion
    }
}
